# Templ8

A powerful, extensible templating engine.

## Status

| Source     | Shields                                                                                                                         |
| ---------- | ------------------------------------------------------------------------------------------------------------------------------- |
| Project    | ![release][release_shield] ![license][license_shield] ![lines][lines_shield] ![languages][languages_shield]                     |
| Health     | ![codacy][codacy_shield] ![readthedocs][readthedocs_shield] ![travis][travis_shield] ![codacy_coverage][codacy_coverage_shield] |
| Repository | ![issues][issues_shield] ![issues_closed][issues_closed_shield] ![pulls][pulls_shield] ![pulls_closed][pulls_closed_shield]     |
| Publishers | ![pypi][pypi_shield] ![python_versions][python_versions_shield] ![pypi_downloads][pypi_downloads_shield]                        |
| Activity   | ![contributors][contributors_shield] ![monthly_commits][monthly_commits_shield] ![last_commit][last_commit_shield]              |

## Installing

```bash
pip install templ8
```

## Usage

```bash
usage: templ8 [-h] [-v] [-d] [-s] input

Templ8 cli version: 1.4.1

positional arguments:
  input          input file path

optional arguments:
  -h, --help     show this help message and exit
  -v, --verbose  increase the verbosity level
  -d, --dry-run  don't generate anything
  -s, --silent   don't output any logs
```

### Features

The engine can:

-   Parse Jinja templates.
-   Copy static files.
-   Set default values for render context.
-   Dynamically rename folders after generation.
-   Run initialization scripts.

You can specify:

-   Files to include.
-   Render context values.
-   Protected files not to touch.
-   Collection sources to search.
-   Loader paths for jinja include tags.
-   Whether to clear top level files.
-   To allow logical input sub-grouping.

### Example

**`.template.yml`**:

```yml
output_dir: .

clear_top_level: true
logical_grouping: true

collections:
    - common
    - python

loader_paths:
    - docs

protected:
    - .prettierignore
    - Gruntfile.js
    - MANIFEST.in

render_context:
    project:
        project_name: templ8
        version: 1.4.1

        verbose_name: Templ8
        description: A powerful, extensible templating engine.

        repository: https://github.com/JoelLefkowitz/templ8
        codacy_id: f407d8c4db8c4b83877bca0543aba744

        inserts:
            - usage.md

    author:
        author: Joel Lefkowitz
        author_email: joellefkowitz@hotmail.com
        author_github_handle: joellefkowitz

    package:
        is_pypi_package: true
        package_name: templ8

        console_scripts:
            - 'templ8 = templ8.__main__:main'

        install_requires:
            - art
            - configparser
            - dataclasses
            - jinja2schema
```

```bash

 .-') _       ('-.    _   .-')       _ (`-.
(  OO) )    _(  OO)  ( '.( OO )_    ( (OO  )
/     '._  (,------.  ,--.   ,--.) _.`     \  ,--.        .-----.
|'--...__)  |  .---'  |   `.'   | (__...--''  |  |.-')   /  .-.  \
'--.  .--'  |  |      |         |  |  /  | |  |  | OO ) |   \_.' /
   |  |    (|  '--.   |  |'.'|  |  |  |_.' |  |  |`-' |  /  .-. '.
   |  |     |  .--'   |  |   |  |  |  .___.' (|  '---.' |  |   |  |
   |  |     |  `---.  |  |   |  |  |  |       |      |   \  '-'  /
   `--'     `------'  `--'   `--'  `--'       `------'    `----''


Loader

collection_sources
  ['/Users/joel/Workspace/templ8/templ8/core']

collection_names
  ['common', 'python']

output_dir
  '.'

includes
  []

protected
  ['.template.yml',
   '.prettierignore',
   'Gruntfile.js',
   'MANIFEST.in',
   'setup.cfg',
   '.gitignore',
   '.cspell.txt',
   'CHANGELOG.md',
   '.pylintrc']

Clearing top level files
  Leaving Gruntfile.js unchanged
  Deleting LICENSE.md
  Leaving .pylintrc unchanged
  Deleting .mypy.ini
  Deleting .cspell.json
  Leaving CHANGELOG.md unchanged
  Deleting .bandit
  Deleting .bumpversion.cfg
  Deleting .quickdocs.yml
  Deleting .readthedocs.yml
  Leaving .prettierignore unchanged
  Deleting README.md
  Leaving .cspell.txt unchanged
  Deleting .prettierrc.yml
  Deleting setup.py
  Leaving .gitignore unchanged
  Deleting package.json
  Deleting CONTRIBUTING.md
  Deleting tox.ini
  Leaving setup.cfg unchanged
  Deleting .remarkrc
  Deleting .travis.yml
  Deleting .remarkignore
  Leaving .template.yml unchanged

Processing collection: common
  Leaving ./Gruntfile.js unchanged
  Copying ./.cspell.json
  Leaving ./CHANGELOG.md unchanged
  Copying ./.readthedocs.yml
  Leaving ./.prettierignore unchanged
  Leaving ./.cspell.txt unchanged
  Copying ./.prettierrc.yml
  Leaving ./.gitignore unchanged
  Copying ./package.json
  Copying ./CONTRIBUTING.md
  Copying ./.remarkrc
  Copying ./.travis.yml
  Copying ./.remarkignore
  Templating ./.quickdocs.yml
  Templating ./.bumpversion.cfg
  Templating ./LICENSE.md
  Templating ./README.md
  Running quickdocs .quickdocs.yml in .

Processing collection: python
  Leaving ./.pylintrc unchanged
  Copying ./.mypy.ini
  Copying ./.bandit
  Copying ./tests/__init__.py
  Copying ./templ8/__init__.py
  Leaving ./setup.cfg unchanged
  Templating ./setup.py
  Templating ./tox.ini
  Templating ./.travis.yml
  Leaving ./Gruntfile.js unchanged
  Running python -m venv venv in .
```

### Plugins

All collection sources contain a specification file at their root.

**`metadata.json`**:

```json
{
    "default_variables": {
        "tox_envlist": ["py36", "py37", "py38", "py39"],
        "frameworks": [],
        "install_requires": [],
        "is_portfolio": false,
        "keywords": [],
        "pytest_ignore_glob": null,
        "python_requires": 3.6
    },
    "default_protected": [".pylintrc"],
    "dynamic_folders": [
        {
            "segment": "src",
            "token": "package_name"
        }
    ],
    "initialization": ["python -m venv venv"]
}
```

## Tests

To run unit tests and generate a coverage report:

```bash
grunt tests:unit
```

## Documentation

This repository's documentation is hosted on [readthedocs][readthedocs].

To generate the sphinx configuration:

```bash
grunt docs:generate
```

Then build the documentation:

```bash
grunt docs:build
```

## Tooling

To run linters:

```bash
grunt lint
```

To run formatters:

```bash
grunt format
```

Before committing new code:

```bash
grunt precommit
```

This will run linters, formatters, tests, generate a test coverage report and the sphinx configuration.

## Continuous integration

This repository uses Travis CI to build and test each commit. Formatting tasks and writing/generating documentation must be done before committing new code.

## Versioning

This repository adheres to semantic versioning standards.
For more information on semantic versioning visit [SemVer][semver].

Bump2version is used to version and tag changes.
For example:

```bash
bump2version patch
```

## Changelog

Please read this repository's [CHANGELOG](CHANGELOG.md) for details on changes that have been made.

## Contributing

Please read this repository's guidelines on [CONTRIBUTING](CONTRIBUTING.md) for details on our code of conduct and the process for submitting pull requests.

## Contributors

-   **Joel Lefkowitz** - _Initial work_ - [Joel Lefkowitz][author]

[![Buy Me A Coffee][coffee_button]][coffee]

## Remarks

Lots of love to the open source community!

![Be kind][be_kind]

<!-- Public links -->

[semver]: http://semver.org/

<!-- External links -->

[readthedocs]: https://templ8.readthedocs.io/en/latest/
[coffee]: https://www.buymeacoffee.com/joellefkowitz
[coffee_button]: https://cdn.buymeacoffee.com/buttons/default-blue.png
[be_kind]: https://media.giphy.com/media/osAcIGTSyeovPq6Xph/giphy.gif

<!-- Acknowledgments -->

[author]: https://github.com/joellefkowitz

<!-- Project shields -->

[release_shield]: https://img.shields.io/github/v/tag/joellefkowitz/templ8
[license_shield]: https://img.shields.io/github/license/joellefkowitz/templ8
[lines_shield]: https://img.shields.io/tokei/lines/github/joellefkowitz/templ8
[languages_shield]: https://img.shields.io/github/languages/count/joellefkowitz/templ8

<!-- Health shields -->

[codacy_shield]: https://img.shields.io/codacy/grade/f407d8c4db8c4b83877bca0543aba744
[readthedocs_shield]: https://img.shields.io/readthedocs/templ8
[travis_shield]: https://img.shields.io/travis/com/joellefkowitz/templ8
[codacy_coverage_shield]: https://img.shields.io/codacy/coverage/f407d8c4db8c4b83877bca0543aba744

<!-- Repository shields -->

[issues_shield]: https://img.shields.io/github/issues/joellefkowitz/templ8
[issues_closed_shield]: https://img.shields.io/github/issues-closed/joellefkowitz/templ8
[pulls_shield]: https://img.shields.io/github/issues-pr/joellefkowitz/templ8
[pulls_closed_shield]: https://img.shields.io/github/issues-pr-closed/joellefkowitz/templ8

<!-- Publishers shields -->

[pypi_shield]: https://img.shields.io/pypi/v/templ8
[python_versions_shield]: https://img.shields.io/pypi/pyversions/templ8
[pypi_downloads_shield]: https://img.shields.io/pypi/dw/templ8

<!-- Activity shields -->

[contributors_shield]: https://img.shields.io/github/contributors/joellefkowitz/templ8
[monthly_commits_shield]: https://img.shields.io/github/commit-activity/m/joellefkowitz/templ8
[last_commit_shield]: https://img.shields.io/github/last-commit/joellefkowitz/templ8
